@@ -16,6 +16,8 @@ import binascii
 import collections
 
 import netaddr
+from neutron_lib import constants as lib_constants
+from neutron_lib import exceptions
 from oslo_log import log as logging
 from oslo_utils import excutils
 import six
@@ -24,8 +26,7 @@ from neutron._i18n import _LE, _LW
 from neutron.agent.l3 import dvr_fip_ns
 from neutron.agent.l3 import dvr_router_base
 from neutron.agent.linux import ip_lib
-from neutron.common import constants as l3_constants
-from neutron.common import exceptions
+from neutron.common import constants as n_const
 from neutron.common import utils as common_utils
 
 LOG = logging.getLogger(__name__)
@@ -55,40 +56,45 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                    (i['host'] == self.host) or
                    (i.get('dest_host') == self.host))]
 
-    def _handle_fip_nat_rules(self, interface_name):
-        """Configures NAT rules for Floating IPs for DVR."""
-        self.iptables_manager.ipv4['nat'].empty_chain('POSTROUTING')
-        self.iptables_manager.ipv4['nat'].empty_chain('snat')
+    def floating_forward_rules(self, floating_ip, fixed_ip):
+        """Override this function defined in router_info for dvr routers."""
+        if not self.fip_ns:
+            return []
 
-        # Add back the jump to float-snat
-        self.iptables_manager.ipv4['nat'].add_rule('snat', '-j $float-snat')
+        rtr_2_fip_name = self.fip_ns.get_rtr_ext_device_name(self.router_id)
+        dnat_from_floatingip_to_fixedip = (
+            'PREROUTING', '-d %s/32 -i %s -j DNAT --to-destination %s' % (
+                floating_ip, rtr_2_fip_name, fixed_ip))
+        snat_from_fixedip_to_floatingip = (
+            'float-snat', '-s %s/32 -j SNAT --to-source %s' % (
+                fixed_ip, floating_ip))
+        return [dnat_from_floatingip_to_fixedip,
+                snat_from_fixedip_to_floatingip]
 
-        # And add the NAT rule back
-        rule = ('POSTROUTING', '! -i %(interface_name)s '
-                '! -o %(interface_name)s -m conntrack ! '
-                '--ctstate DNAT -j ACCEPT' %
-                {'interface_name': interface_name})
-        self.iptables_manager.ipv4['nat'].add_rule(*rule)
+    def floating_mangle_rules(self, floating_ip, fixed_ip, internal_mark):
+        if not self.fip_ns:
+            return []
 
-        self.iptables_manager.apply()
+        rtr_2_fip_name = self.fip_ns.get_rtr_ext_device_name(self.router_id)
+        mark_traffic_to_floating_ip = (
+            'floatingip', '-d %s/32 -i %s -j MARK --set-xmark %s' % (
+                floating_ip, rtr_2_fip_name, internal_mark))
+        mark_traffic_from_fixed_ip = (
+            'FORWARD', '-s %s/32 -j $float-snat' % fixed_ip)
+        return [mark_traffic_to_floating_ip, mark_traffic_from_fixed_ip]
 
     def floating_ip_added_dist(self, fip, fip_cidr):
         """Add floating IP to FIP namespace."""
         floating_ip = fip['floating_ip_address']
         fixed_ip = fip['fixed_ip_address']
-        rule_pr = self.fip_ns.allocate_rule_priority(floating_ip)
-        self.floating_ips_dict[floating_ip] = rule_pr
+        self._add_floating_ip_rule(floating_ip, fixed_ip)
         fip_2_rtr_name = self.fip_ns.get_int_device_name(self.router_id)
-        ip_rule = ip_lib.IPRule(namespace=self.ns_name)
-        ip_rule.rule.add(ip=fixed_ip,
-                         table=dvr_fip_ns.FIP_RT_TBL,
-                         priority=rule_pr)
         #Add routing rule in fip namespace
         fip_ns_name = self.fip_ns.get_name()
         if self.rtr_fip_subnet is None:
             self.rtr_fip_subnet = self.fip_ns.local_subnets.allocate(
                 self.router_id)
-        rtr_2_fip, _ = self.rtr_fip_subnet.get_pair()
+        rtr_2_fip, __ = self.rtr_fip_subnet.get_pair()
         device = ip_lib.IPDevice(fip_2_rtr_name, namespace=fip_ns_name)
         device.route.add_route(fip_cidr, str(rtr_2_fip.ip))
         interface_name = (
@@ -101,6 +107,24 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         # update internal structures
         self.dist_fip_count = self.dist_fip_count + 1
 
+    def _add_floating_ip_rule(self, floating_ip, fixed_ip):
+        rule_pr = self.fip_ns.allocate_rule_priority(floating_ip)
+        self.floating_ips_dict[floating_ip] = rule_pr
+        ip_rule = ip_lib.IPRule(namespace=self.ns_name)
+        ip_rule.rule.add(ip=fixed_ip,
+                         table=dvr_fip_ns.FIP_RT_TBL,
+                         priority=rule_pr)
+
+    def _remove_floating_ip_rule(self, floating_ip):
+        if floating_ip in self.floating_ips_dict:
+            rule_pr = self.floating_ips_dict[floating_ip]
+            ip_rule = ip_lib.IPRule(namespace=self.ns_name)
+            ip_rule.rule.delete(ip=floating_ip,
+                                table=dvr_fip_ns.FIP_RT_TBL,
+                                priority=rule_pr)
+            self.fip_ns.deallocate_rule_priority(floating_ip)
+            #TODO(rajeev): Handle else case - exception/log?
+
     def floating_ip_removed_dist(self, fip_cidr):
         """Remove floating IP from FIP namespace."""
         floating_ip = fip_cidr.split('/')[0]
@@ -112,14 +136,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
         rtr_2_fip, fip_2_rtr = self.rtr_fip_subnet.get_pair()
         fip_ns_name = self.fip_ns.get_name()
-        if floating_ip in self.floating_ips_dict:
-            rule_pr = self.floating_ips_dict[floating_ip]
-            ip_rule = ip_lib.IPRule(namespace=self.ns_name)
-            ip_rule.rule.delete(ip=floating_ip,
-                                table=dvr_fip_ns.FIP_RT_TBL,
-                                priority=rule_pr)
-            self.fip_ns.deallocate_rule_priority(floating_ip)
-            #TODO(rajeev): Handle else case - exception/log?
+        self._remove_floating_ip_rule(floating_ip)
 
         device = ip_lib.IPDevice(fip_2_rtr_name, namespace=fip_ns_name)
 
@@ -136,22 +153,28 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
             self.rtr_fip_subnet = None
             ns_ip.del_veth(fip_2_rtr_name)
 
-    def add_floating_ip(self, fip, interface_name, device):
-        if not self._add_fip_addr_to_device(fip, device):
-            return l3_constants.FLOATINGIP_STATUS_ERROR
+    def floating_ip_moved_dist(self, fip):
+        """Handle floating IP move between fixed IPs."""
+        floating_ip = fip['floating_ip_address']
+        self._remove_floating_ip_rule(floating_ip)
+        self._add_floating_ip_rule(floating_ip, fip['fixed_ip_address'])
 
+    def add_floating_ip(self, fip, interface_name, device):
         # Special Handling for DVR - update FIP namespace
         ip_cidr = common_utils.ip_to_cidr(fip['floating_ip_address'])
         self.floating_ip_added_dist(fip, ip_cidr)
-        return l3_constants.FLOATINGIP_STATUS_ACTIVE
+        return lib_constants.FLOATINGIP_STATUS_ACTIVE
 
     def remove_floating_ip(self, device, ip_cidr):
-        super(DvrLocalRouter, self).remove_floating_ip(device, ip_cidr)
         self.floating_ip_removed_dist(ip_cidr)
 
+    def move_floating_ip(self, fip):
+        self.floating_ip_moved_dist(fip)
+        return lib_constants.FLOATINGIP_STATUS_ACTIVE
+
     def _get_internal_port(self, subnet_id):
         """Return internal router port based on subnet_id."""
-        router_ports = self.router.get(l3_constants.INTERFACE_KEY, [])
+        router_ports = self.router.get(lib_constants.INTERFACE_KEY, [])
         for port in router_ports:
             fips = port['fixed_ips']
             for f in fips:
@@ -229,7 +252,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         subnet_ports = self.agent.get_ports_by_subnet(subnet_id)
 
         for p in subnet_ports:
-            if p['device_owner'] not in l3_constants.ROUTER_INTERFACE_OWNERS:
+            if p['device_owner'] not in lib_constants.ROUTER_INTERFACE_OWNERS:
                 for fixed_ip in p['fixed_ips']:
                     self._update_arp_entry(fixed_ip['ip_address'],
                                            p['mac_address'],
@@ -269,6 +292,32 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         except exceptions.DeviceNotFoundError:
             pass
 
+    def _stale_ip_rule_cleanup(self, ns_ipr, ns_ipd, ip_version):
+        ip_rules_list = ns_ipr.rule.list_rules(ip_version)
+        snat_table_list = []
+        for ip_rule in ip_rules_list:
+            snat_table = ip_rule['table']
+            priority = ip_rule['priority']
+            if snat_table in ['local', 'default', 'main']:
+                continue
+            if (ip_version == lib_constants.IP_VERSION_4 and
+                snat_table in range(dvr_fip_ns.FIP_PR_START,
+                                    dvr_fip_ns.FIP_PR_END)):
+                continue
+            gateway_cidr = ip_rule['from']
+            ns_ipr.rule.delete(ip=gateway_cidr,
+                               table=snat_table,
+                               priority=priority)
+            snat_table_list.append(snat_table)
+        for tb in snat_table_list:
+            ns_ipd.route.flush(ip_version, table=tb)
+
+    def gateway_redirect_cleanup(self, rtr_interface):
+        ns_ipr = ip_lib.IPRule(namespace=self.ns_name)
+        ns_ipd = ip_lib.IPDevice(rtr_interface, namespace=self.ns_name)
+        self._stale_ip_rule_cleanup(ns_ipr, ns_ipd, lib_constants.IP_VERSION_4)
+        self._stale_ip_rule_cleanup(ns_ipr, ns_ipd, lib_constants.IP_VERSION_6)
+
     def _snat_redirect_modify(self, gateway, sn_port, sn_int, is_add):
         """Adds or removes rules and routes for SNAT redirection."""
         try:
@@ -364,7 +413,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def get_floating_agent_gw_interface(self, ext_net_id):
         """Filter Floating Agent GW port for the external network."""
-        fip_ports = self.router.get(l3_constants.FLOATINGIP_AGENT_INTF_KEY, [])
+        fip_ports = self.router.get(n_const.FLOATINGIP_AGENT_INTF_KEY, [])
         return next(
             (p for p in fip_ports if p['network_id'] == ext_net_id), None)
 
@@ -403,21 +452,46 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
             to_fip_interface_name = (
                 self.get_external_device_interface_name(ex_gw_port))
             self.process_floating_ip_addresses(to_fip_interface_name)
-        for p in self.internal_ports:
-            # NOTE: When removing the gateway port, pass in the snat_port
-            # cache along with the current ports.
-            gateway = self.get_snat_port_for_internal_port(p, self.snat_ports)
-            if not gateway:
-                continue
-            internal_interface = self.get_internal_device_name(p['id'])
-            self._snat_redirect_remove(gateway, p, internal_interface)
+        # NOTE:_snat_redirect_remove should be only called when the
+        # gateway is cleared and should not be called when the gateway
+        # is moved or rescheduled.
+        if not self.router.get('gw_port'):
+            for p in self.internal_ports:
+                # NOTE: When removing the gateway port, pass in the snat_port
+                # cache along with the current ports.
+                gateway = self.get_snat_port_for_internal_port(
+                    p, self.snat_ports)
+                if not gateway:
+                    continue
+                internal_interface = self.get_internal_device_name(p['id'])
+                self._snat_redirect_remove(gateway, p, internal_interface)
 
     def _handle_router_snat_rules(self, ex_gw_port, interface_name):
-        pass
+        """Configures NAT rules for Floating IPs for DVR."""
+
+        self.iptables_manager.ipv4['nat'].empty_chain('POSTROUTING')
+        self.iptables_manager.ipv4['nat'].empty_chain('snat')
+
+        ex_gw_port = self.get_ex_gw_port()
+        if not ex_gw_port:
+            return
+
+        ext_device_name = self.get_external_device_interface_name(ex_gw_port)
+        floatingips = self.get_floating_ips()
+        if not ext_device_name or not floatingips:
+            # Without router to fip device, or without any floating ip,
+            # the snat rules should not be added
+            return
+
+        # Add back the jump to float-snat
+        self.iptables_manager.ipv4['nat'].add_rule('snat', '-j $float-snat')
+
+        rule = self._prevent_snat_for_internal_traffic_rule(ext_device_name)
+        self.iptables_manager.ipv4['nat'].add_rule(*rule)
 
     def _get_address_scope_mark(self):
         # Prepare address scope iptables rule for internal ports
-        internal_ports = self.router.get(l3_constants.INTERFACE_KEY, [])
+        internal_ports = self.router.get(lib_constants.INTERFACE_KEY, [])
         ports_scopemark = self._get_port_devicename_scopemark(
             internal_ports, self.get_internal_device_name)
         # DVR local router will use rfp port as external port
@@ -431,7 +505,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
         ext_scope = self._get_external_address_scope()
         ext_scope_mark = self.get_address_scope_mark_mask(ext_scope)
-        ports_scopemark[l3_constants.IP_VERSION_4][ext_device_name] = (
+        ports_scopemark[lib_constants.IP_VERSION_4][ext_device_name] = (
             ext_scope_mark)
         return ports_scopemark
 
@@ -475,9 +549,27 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                 (self.dist_fip_count == 0)):
                 self.fip_ns.create_rtr_2_fip_link(self)
 
-                # kicks the FW Agent to add rules for the IR namespace if
-                # configured
-                self.agent.process_router_add(self)
+    def get_router_cidrs(self, device):
+        """As no floatingip will be set on the rfp device. Get floatingip from
+        the route of fip namespace.
+        """
+        if not self.fip_ns:
+            return set()
+
+        fip_ns_name = self.fip_ns.get_name()
+        fip_2_rtr_name = self.fip_ns.get_int_device_name(self.router_id)
+        device = ip_lib.IPDevice(fip_2_rtr_name, namespace=fip_ns_name)
+        if not device.exists():
+            return set()
+
+        if self.rtr_fip_subnet is None:
+            self.rtr_fip_subnet = self.fip_ns.local_subnets.allocate(
+                self.router_id)
+        rtr_2_fip, _fip_2_rtr = self.rtr_fip_subnet.get_pair()
+        exist_routes = device.route.list_routes(
+            lib_constants.IP_VERSION_4, via=str(rtr_2_fip.ip))
+        return {common_utils.ip_to_cidr(route['cidr'])
+                for route in exist_routes}
 
     def process(self, agent):
         ex_gw_port = self.get_ex_gw_port()
