@@ -19,6 +19,7 @@ import functools
 import itertools
 import random
 
+from neutron_lib import constants as lib_const
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
@@ -107,9 +108,11 @@ class L3Scheduler(object):
             filters = {'id': router_ids,
                        'status': [constants.ROUTER_STATUS_ACTIVE]}
             routers = plugin.get_routers(context, filters=filters)
-            return self._filter_unscheduled_routers(context, plugin, routers)
+            result = self._filter_unscheduled_routers(context, plugin, routers)
         else:
-            return self._get_unscheduled_routers(context, plugin)
+            result = self._get_unscheduled_routers(context, plugin)
+        return [r for r in result
+                if plugin.router_supports_scheduling(context, r['id'])]
 
     def _get_routers_can_schedule(self, context, plugin, routers, l3_agent):
         """Get the subset of routers that can be scheduled on the L3 agent."""
@@ -135,27 +138,27 @@ class L3Scheduler(object):
         :returns: True if routers have been successfully assigned to host
         """
         l3_agent = plugin.get_enabled_agent_on_host(
-            context, constants.AGENT_TYPE_L3, host)
+            context, lib_const.AGENT_TYPE_L3, host)
         if not l3_agent:
-            return False
+            return
 
         unscheduled_routers = self._get_routers_to_schedule(
             context, plugin, router_ids)
         if not unscheduled_routers:
             if utils.is_extension_supported(
-                    plugin, constants.L3_HA_MODE_EXT_ALIAS):
-                return self._schedule_ha_routers_to_additional_agent(
+                    plugin, lib_const.L3_HA_MODE_EXT_ALIAS):
+                self._schedule_ha_routers_to_additional_agent(
                     plugin, context, l3_agent)
+                return
 
         target_routers = self._get_routers_can_schedule(
             context, plugin, unscheduled_routers, l3_agent)
         if not target_routers:
             LOG.warning(_LW('No routers compatible with L3 agent '
                             'configuration on host %s'), host)
-            return False
+            return
 
         self._bind_routers(context, plugin, target_routers, l3_agent)
-        return True
 
     def _get_candidates(self, plugin, context, sync_router):
         """Return L3 agents where a router could be scheduled."""
@@ -197,19 +200,30 @@ class L3Scheduler(object):
             else:
                 self.bind_router(context, router['id'], l3_agent)
 
-    def bind_router(self, context, router_id, chosen_agent):
+    def bind_router(self, context, router_id, chosen_agent,
+                    binding_index=l3_agentschedulers_db.LOWEST_BINDING_INDEX):
         """Bind the router to the l3 agent which has been chosen."""
+        # Pre-cache the agent's id so that if an exception is raised we can
+        # safely access its value. Otherwise, sqlalchemy will try to fetch it
+        # from the database during a rollback, which is bad for us.
+        agent_id = chosen_agent.id
+
         try:
             with context.session.begin(subtransactions=True):
                 binding = l3_agentschedulers_db.RouterL3AgentBinding()
                 binding.l3_agent = chosen_agent
                 binding.router_id = router_id
+                binding.binding_index = binding_index
                 context.session.add(binding)
-        except db_exc.DBDuplicateEntry:
+        except db_exc.DBDuplicateEntry as error:
             LOG.debug('Router %(router_id)s has already been scheduled '
-                      'to L3 agent %(agent_id)s.',
-                      {'agent_id': chosen_agent.id,
-                       'router_id': router_id})
+                      'to L3 agent %(agent_id)s (tried to bind with '
+                      'binding_index %(binding_index)d). The conflict was '
+                      'with columns %(columns)r.',
+                      {'agent_id': agent_id,
+                       'router_id': router_id,
+                       'binding_index': binding_index,
+                       'columns': error.columns})
             return
         except db_exc.DBReferenceError:
             LOG.debug('Router %s has already been removed '
@@ -217,11 +231,16 @@ class L3Scheduler(object):
             return
 
         LOG.debug('Router %(router_id)s is scheduled to L3 agent '
-                  '%(agent_id)s', {'router_id': router_id,
-                                   'agent_id': chosen_agent.id})
+                  '%(agent_id)s with binding_index %(binding_index)d',
+                  {'router_id': router_id,
+                   'agent_id': agent_id,
+                   'binding_index': binding_index})
+        return binding
 
     def _schedule_router(self, plugin, context, router_id,
                          candidates=None):
+        if not plugin.router_supports_scheduling(context, router_id):
+            return
         sync_router = plugin.get_router(context, router_id)
         candidates = candidates or self._get_candidates(
             plugin, context, sync_router)
@@ -266,7 +285,7 @@ class L3Scheduler(object):
                                   tenant_id)
 
     def create_ha_port_and_bind(self, plugin, context, router_id,
-                                tenant_id, agent):
+                                tenant_id, agent, is_manual_scheduling=False):
         """Creates and binds a new HA port for this agent."""
         ctxt = context.elevated()
         creator = functools.partial(self._add_port_from_net,
@@ -274,21 +293,38 @@ class L3Scheduler(object):
         dep_getter = functools.partial(plugin.get_ha_network, ctxt, tenant_id)
         dep_creator = functools.partial(plugin._create_ha_network,
                                         ctxt, tenant_id)
+        dep_deleter = functools.partial(plugin._delete_ha_network, ctxt)
         dep_id_attr = 'network_id'
-        try:
-            port_binding = utils.create_object_with_dependency(
-                creator, dep_getter, dep_creator, dep_id_attr)[0]
-            with db_api.autonested_transaction(context.session):
-                port_binding.l3_agent_id = agent['id']
-        except db_exc.DBDuplicateEntry:
-            LOG.debug("Router %(router)s already scheduled for agent "
-                      "%(agent)s", {'router': router_id, 'agent': agent['id']})
-        except l3.RouterNotFound:
-            LOG.debug('Router %s has already been removed '
-                      'by concurrent operation', router_id)
-            return
 
-        self.bind_router(context, router_id, agent)
+        for attempts in range(1, db_api.MAX_RETRIES + 1):
+            binding_index = plugin.get_vacant_binding_index(
+                context, router_id, is_manual_scheduling)
+            if binding_index == -1:
+                LOG.debug("Couldn't find a vacant binding_index for router %s",
+                          router_id)
+                return
+
+            # This might fail in case of concurrent calls, which is good for us
+            # as we can skip the rest of this function.
+            if not self.bind_router(context, router_id, agent, binding_index):
+                return
+
+            try:
+                port_binding = utils.create_object_with_dependency(
+                    creator, dep_getter, dep_creator,
+                    dep_id_attr, dep_deleter)[0]
+                with db_api.autonested_transaction(context.session):
+                    port_binding.l3_agent_id = agent['id']
+                return
+            except db_exc.DBDuplicateEntry:
+                LOG.debug("Router %(router)s already scheduled for agent "
+                          "%(agent)s", {'router': router_id,
+                                        'agent': agent['id']})
+                return
+            except l3.RouterNotFound:
+                LOG.debug('Router %s has already been removed '
+                          'by concurrent operation', router_id)
+                return
 
     def get_ha_routers_l3_agents_counts(self, context, plugin, filters=None):
         """Return a mapping (router, # agents) matching specified filters."""
@@ -304,31 +340,34 @@ class L3Scheduler(object):
 
         routers_agents = self.get_ha_routers_l3_agents_counts(context, plugin,
                                                               agent)
-        scheduled = False
         admin_ctx = context.elevated()
-        for router, agents in routers_agents:
-            max_agents_not_reached = (
-                not self.max_ha_agents or agents < self.max_ha_agents)
-            if max_agents_not_reached:
-                if not self._router_has_binding(admin_ctx, router['id'],
-                                                agent.id):
-                    self.create_ha_port_and_bind(plugin, admin_ctx,
-                                                 router['id'],
-                                                 router['tenant_id'],
-                                                 agent)
-                    scheduled = True
-
-        return scheduled
+        underscheduled_routers = [router for router, agents in routers_agents
+                                  if (not self.max_ha_agents or
+                                      agents < self.max_ha_agents)]
+        schedulable_routers = self._get_routers_can_schedule(
+            admin_ctx, plugin, underscheduled_routers, agent)
+        for router in schedulable_routers:
+            if not self._router_has_binding(admin_ctx, router['id'],
+                                            agent.id):
+                self.create_ha_port_and_bind(plugin, admin_ctx,
+                                             router['id'],
+                                             router['tenant_id'],
+                                             agent)
 
     def _bind_ha_router_to_agents(self, plugin, context, router_id,
                                  chosen_agents):
         port_bindings = plugin.get_ha_router_port_bindings(context,
                                                            [router_id])
-        for port_binding, agent in zip(port_bindings, chosen_agents):
+        binding_indices = range(l3_agentschedulers_db.LOWEST_BINDING_INDEX,
+                                len(port_bindings) + 1)
+        for port_binding, agent, binding_index in zip(
+            port_bindings, chosen_agents, binding_indices):
+
+            if not self.bind_router(context, router_id, agent, binding_index):
+                continue
             try:
                 with db_api.autonested_transaction(context.session):
                     port_binding.l3_agent_id = agent.id
-                    self.bind_router(context, router_id, agent)
             except db_exc.DBDuplicateEntry:
                 LOG.debug("Router %(router)s already scheduled for agent "
                           "%(agent)s", {'router': router_id,
