@@ -25,7 +25,6 @@ from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
 from neutron.common import utils
 from neutron.tests.common import net_helpers
-from neutron.tests.functional.agent.linux import base
 from neutron.tests.functional import base as functional_base
 
 LOG = logging.getLogger(__name__)
@@ -52,11 +51,11 @@ class IpLibTestFramework(functional_base.BaseSudoTestCase):
 
     def generate_device_details(self, name=None, ip_cidrs=None,
                                 mac_address=None, namespace=None):
-        return Device(name or base.get_rand_name(),
+        return Device(name or utils.get_rand_name(),
                       ip_cidrs or ["%s/24" % TEST_IP],
                       mac_address or
                       utils.get_random_mac('fa:16:3e:00:00:00'.split(':')),
-                      namespace or base.get_rand_name())
+                      namespace or utils.get_rand_name())
 
     def _safe_delete_device(self, device):
         try:
@@ -97,6 +96,9 @@ class IpLibTestCase(IpLibTestFramework):
         self.assertTrue(
             ip_lib.device_exists(device.name, namespace=attr.namespace))
 
+        self.assertFalse(
+            ip_lib.device_exists(attr.name, namespace='wrong_namespace'))
+
         device.link.delete()
 
         self.assertFalse(
@@ -182,7 +184,7 @@ class IpLibTestCase(IpLibTestFramework):
 
     def test_dummy_exists(self):
         namespace = self.useFixture(net_helpers.NamespaceFixture())
-        dev_name = base.get_rand_name()
+        dev_name = utils.get_rand_name()
         device = namespace.ip_wrapper.add_dummy(dev_name)
         self.addCleanup(self._safe_delete_device, device)
         self._check_for_device_name(namespace.ip_wrapper, dev_name, True)
